;;; custom-load.el --- automatically extracted custom dependencies

;;; Code:

(autoload 'custom-add-loads "cus-load")

(custom-add-loads 'reftex-finding-files '("reftex-vars"))
(custom-add-loads 'reftex-defining-label-environments '("reftex-vars"))
(custom-add-loads 'tex '("reftex-vars"))
(custom-add-loads 'reftex '("reftex-vars"))
(custom-add-loads 'reftex-label-support '("reftex-vars"))
(custom-add-loads 'reftex-index-support '("reftex-vars"))
(custom-add-loads 'reftex-miscellaneous-configurations '("reftex-vars"))
(custom-add-loads 'reftex-citation-support '("reftex-vars"))
(custom-add-loads 'reftex-table-of-contents-browser '("reftex-vars"))
(custom-add-loads 'reftex-making-and-inserting-labels '("reftex-vars"))
(custom-add-loads 'reftex-referencing-labels '("reftex-vars"))
(custom-add-loads 'reftex-fontification-configurations '("reftex-vars"))
(custom-add-loads 'reftex-viewing-cross-references '("reftex-vars"))
(custom-add-loads 'reftex-optimizations-for-large-documents '("reftex-vars"))
(custom-add-loads 'LaTeX '("reftex-vars"))

;;; custom-load.el ends here
